@@ -20,7 +20,6 @@ package s3utils
 import (
 	"errors"
 	"net/url"
-	"reflect"
 	"testing"
 )
 
@@ -298,81 +297,6 @@ func TestQueryEncode(t *testing.T) {
 	}
 }
 
-// Tests tag decode to map
-func TestTagDecode(t *testing.T) {
-	testCases := []struct {
-		// canonical input
-		canonicalInput string
-
-		// Expected result.
-		resultMap map[string]string
-	}{
-		{"k=thisisthe%25url", map[string]string{"k": "thisisthe%url"}},
-		{"k=%E6%9C%AC%E8%AA%9E", map[string]string{"k": "本語"}},
-		{"k=%E6%9C%AC%E8%AA%9E.1", map[string]string{"k": "本語.1"}},
-		{"k=%3E123", map[string]string{"k": ">123"}},
-		{"k=myurl%23link", map[string]string{"k": "myurl#link"}},
-		{"k=space%20in%20url", map[string]string{"k": "space in url"}},
-		{"k=url%2Bpath", map[string]string{"k": "url+path"}},
-		{"k=url%2Fpath", map[string]string{"k": "url/path"}},
-	}
-
-	for _, testCase := range testCases {
-		testCase := testCase
-		t.Run("", func(t *testing.T) {
-			gotResult := TagDecode(testCase.canonicalInput)
-			if !reflect.DeepEqual(testCase.resultMap, gotResult) {
-				t.Errorf("Expected %s, got %s", testCase.resultMap, gotResult)
-			}
-		})
-	}
-}
-
-// Tests tag encode function for user tags.
-func TestTagEncode(t *testing.T) {
-	testCases := []struct {
-		// Input.
-		inputMap map[string]string
-		// Expected result.
-		result string
-	}{
-		{map[string]string{
-			"k": "thisisthe%url",
-		}, "k=thisisthe%25url"},
-		{map[string]string{
-			"k": "本語",
-		}, "k=%E6%9C%AC%E8%AA%9E"},
-		{map[string]string{
-			"k": "本語.1",
-		}, "k=%E6%9C%AC%E8%AA%9E.1"},
-		{map[string]string{
-			"k": ">123",
-		}, "k=%3E123"},
-		{map[string]string{
-			"k": "myurl#link",
-		}, "k=myurl%23link"},
-		{map[string]string{
-			"k": "space in url",
-		}, "k=space%20in%20url"},
-		{map[string]string{
-			"k": "url+path",
-		}, "k=url%2Bpath"},
-		{map[string]string{
-			"k": "url/path",
-		}, "k=url%2Fpath"},
-	}
-
-	for _, testCase := range testCases {
-		testCase := testCase
-		t.Run("", func(t *testing.T) {
-			gotResult := TagEncode(testCase.inputMap)
-			if testCase.result != gotResult {
-				t.Errorf("Expected %s, got %s", testCase.result, gotResult)
-			}
-		})
-	}
-}
-
 // Tests validate the URL path encoder.
 func TestEncodePath(t *testing.T) {
 	testCases := []struct {
@@ -442,7 +366,6 @@ func TestIsValidBucketName(t *testing.T) {
 				t.Errorf("Test %d: Expected to fail with error \"%s\", but instead failed with error \"%s\" instead", i+1, testCase.err.Error(), err.Error())
 			}
 		}
-
 	}
 }
 
@@ -485,6 +408,49 @@ func TestIsValidBucketNameStrict(t *testing.T) {
 				t.Errorf("Test %d: Expected to fail with error \"%s\", but instead failed with error \"%s\" instead", i+1, testCase.err.Error(), err.Error())
 			}
 		}
+	}
+}
 
+func TestIsAmazonPrivateLinkEndpoint(t *testing.T) {
+	testCases := []struct {
+		url string
+		// Expected result.
+		result bool
+	}{
+		{"https://192.168.1.1", false},
+		{"192.168.1.1", false},
+		{"http://storage.googleapis.com", false},
+		{"https://storage.googleapis.com", false},
+		{"storage.googleapis.com", false},
+		{"s3.amazonaws.com", false},
+		{"https://amazons3.amazonaws.com", false},
+		{"-192.168.1.1", false},
+		{"260.192.1.1", false},
+		{"https://s3-.amazonaws.com", false},
+		{"https://s3..amazonaws.com", false},
+		{"https://s3.dualstack.us-west-1.amazonaws.com.cn", false},
+		{"https://s3..us-west-1.amazonaws.com.cn", false},
+		{"https://s3.amazonaws.com", false},
+		{"https://s3-external-1.amazonaws.com", false},
+		{"https://s3.cn-north-1.amazonaws.com.cn", false},
+		{"https://s3-us-west-1.amazonaws.com", false},
+		{"https://s3.us-west-1.amazonaws.com", false},
+		{"https://s3.dualstack.us-west-1.amazonaws.com", false},
+		// valid inputs.
+		{"https://bucket.vpce-1a2b3c4d-5e6f.s3.us-east-1.vpce.amazonaws.com", true},
+		{"https://accesspoint.vpce-1a2b3c4d-5e6f.s3.us-east-1.vpce.amazonaws.com", true},
+		{"https://bucket.vpce-1a2b3c4d-5e6f.s3.us-east-1.vpce.amazonaws.com:443", true},
+		{"https://accesspoint.vpce-1a2b3c4d-5e6f.s3.us-east-1.vpce.amazonaws.com:443", true},
+	}
+
+	for i, testCase := range testCases {
+		u, err := url.Parse(testCase.url)
+		if err != nil {
+			t.Errorf("Test %d: Expected to pass, but failed with: <ERROR> %s", i+1, err)
+		}
+		result := IsAmazonPrivateLinkEndpoint(*u)
+		if testCase.result != result {
+			t.Errorf("Test %d: Expected IsAmazonPrivateLinkEndpoint to be '%v' for input \"%s\", but found it to be '%v' instead", i+1, testCase.result, testCase.url, result)
+		}
 	}
 }
