@@ -20,7 +20,7 @@ package minio
 import (
 	"bytes"
 	"encoding/xml"
-	"io/ioutil"
+	"io"
 	"net/http"
 	"strconv"
 )
@@ -41,7 +41,7 @@ func generateErrorResponse(resp *http.Response, APIErr APIError, bucketName stri
 	encodedErrorResponse := encodeResponse(errorResponse)
 	// write Header.
 	resp.StatusCode = APIErr.HTTPStatusCode
-	resp.Body = ioutil.NopCloser(bytes.NewBuffer(encodedErrorResponse))
+	resp.Body = io.NopCloser(bytes.NewBuffer(encodedErrorResponse))
 
 	return resp
 }
@@ -49,7 +49,7 @@ func generateErrorResponse(resp *http.Response, APIErr APIError, bucketName stri
 // getErrorResponse gets in standard error and resource value and
 // provides a encodable populated response values.
 func getAPIErrorResponse(err APIError, bucketName string) ErrorResponse {
-	var errResp = ErrorResponse{}
+	errResp := ErrorResponse{}
 	errResp.Code = err.Code
 	errResp.Message = err.Description
 	errResp.BucketName = bucketName
